import { Http, I18n, Metadata } from "@zeedhi/core";
import { Util } from "./Util";
import { DialogService, Form, LoadingService, Modal, Select } from "@zeedhi/common";

export class TekLibReport {
    public static validateReportOrParent(reportObj: any, values: any = []): boolean {
        return values.includes(reportObj.value) || values.includes(reportObj.parentReport);
    }

    public static async setModalForm(modal: Modal) {
        modal.show();
        const modalForm = await Util.getAsyncFormInstance('form-modal-report');
        modalForm.clearForm();
        const reportField = Metadata.getInstance<Select>('REPORT');
        const reportModalField = Metadata.getInstance<Select>('REPORT_MODAL');
        const reportData: any = reportField.datasource.data.filter((item) => { return !item.isQr2; });
        reportModalField.datasource.updateData(reportData);
    }

    public static changeReportModal() {
        const modalRow = Metadata.getInstance<Form>('form-modal-report').value;
        const operationField = Metadata.getInstance<Select>('OPERATION');
        if (modalRow.REPORT_MODAL.isCustom) {
            operationField.readonly = false;
            operationField.setValue(null);
            modalRow.NAME = modalRow.REPORT_MODAL.label.replace(`${I18n.translate('CUSTOMIZED')} | `, '');
        } else {
            operationField.readonly = true;
            operationField.setValue('N');
            modalRow.NAME = '';
        }
    }

    public static openReport(route: string, params: any = {}, baseUrl: string = '', isUrl: boolean = false) {
        const reportObj = params.REPORT;
        this.setReportParams(params, reportObj, isUrl);
        const requestParams: any = {
            timeout: 3000000,
            requestType: 'Row',
            row: params,
        };

        LoadingService.show();
        Http.post(route, requestParams).then((response) => {
            const report = response.data ? response.data.messages.pop() : response.messages.pop();
            const isLibOpenReport = report.message.includes('lib_openReport');
            if (isUrl || isLibOpenReport) {
                const url = isLibOpenReport ? `${baseUrl}${report.message}` : report.message;
                const reportOpened = window.open(url);
                if (!reportOpened) {
                    Util.alertMessage('POPUP_BLOCKED', 'error');
                }
            } else {
                if (params.QR2TYPE) {
                    params.__FORMAT = params.QR2TYPE;
                }
                let format = params.__FORMAT ? params.__FORMAT : 'pdf';
                let data = report.message;
                if (data.includes('&format=')) {
                    const splitedData = data.split('&format=');
                    data = splitedData[0];
                    format = splitedData[1];
                }
                this.downloadFile(data, this.transliterateString(reportObj.label), format);
            }
        }).catch((error) => {
            Util.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
        });
    }

    private static setReportParams(params: any, reportObj: any, isUrl: boolean) {
        params.TIMEZONE = params.TIMEZONE || Intl.DateTimeFormat().resolvedOptions().timeZone;
        params.LOCALE = params.LOCALE || this.getReportLanguage();
        params.__report_name = reportObj.value;
        params.isUrl = isUrl;
        params.isCustom = reportObj.isCustom || false;
        params.parentReport = reportObj.parentReport || false;
    }

    private static getReportLanguage() {
        const language = localStorage.getItem('LOGIN_LANGUAGE') || 'pt_br';
        const languageParts = language.split('_');
        languageParts[1] = languageParts[1].toUpperCase();
        return languageParts.join('_');
    }

    public static downloadReport() {
        const form = Metadata.getInstance<Form>('template-form-report');
        const report = form.value.REPORT;
        if (report.isQr2) {
            Util.alertMessage('DOWNLOAD_QR2_REPORT_NOT_ALLOWED', 'error');
        } else {
            DialogService.show({
                name: 'dialog-confirm-download',
                type: 'warning',
                title: I18n.translate('CONFIRMATION'),
                text: I18n.translate('ALERT_CONFIRM_DOWNLOAD_REPORT_FILE'),
                buttons: [
                    {
                        name: 'button-no',
                        component: 'ZdButton',
                        label: I18n.translate('NO'),
                        events: {
                            click: () => DialogService.hide(),
                        },
                        outline: true,
                    },
                    {
                        name: 'button-yes',
                        component: 'ZdButton',
                        label: I18n.translate('YES'),
                        events: {
                            click: () => TekLibReport.downloadReportFile(),
                        },
                    },
                ],
            });
        }
    }

    public static downloadReportFile() {
        const form = Metadata.getInstance<Form>('template-form-report');
        const report = form.value.REPORT;
        const params = {
            FILE_NAME: report.value,
            IS_CUSTOM: report.isCustom || false,
        };
        LoadingService.show();
        Http.post('/lib_getReportContent', params).then((response) => {
            this.downloadFile(response.data.content, this.transliterateString(report.label), response.data.extension);
        }).catch((error) => {
            Util.showErrorMessage(error);
        }).finally(() => {
            DialogService.hide();
            LoadingService.hide();
        });
    }

    public static downloadFile(data: string, fileName: string, extension: string) {
        const a = document.createElement('a');
        document.body.appendChild(a);
        a.setAttribute('style', 'display: none');
        a.href = `data:application/octet-stream;base64,${data}`;
        a.download = `${fileName}.${extension}`;
        a.click();
        Util.showSuccessMessage('SUCCESS_SEND_TO_DOWNLOAD');
    }

    public static transliterateString(text: string) {
        const transliterationTable: any = {
            á: 'a', Á: 'A', à: 'a', À: 'A', a: 'a',
            A: 'A', â: 'a', Â: 'A', å: 'a', Å: 'A',
            ã: 'a', Ã: 'A', ä: 'ae', Ä: 'AE', æ: 'ae',
            Æ: 'AE', c: 'c', C: 'C', ç: 'c', Ç: 'C',
            d: 'd', D: 'D', Ð: 'Dh', ð: 'dh', é: 'e',
            É: 'E', è: 'e', È: 'E', e: 'e', E: 'E',
            ê: 'e', Ê: 'E', ë: 'e', Ë: 'E', ƒ: 'F',
            g: 'g', G: 'G', h: 'h', H: 'H', í: 'i',
            Í: 'I', ì: 'i', Ì: 'I', î: 'i', Î: 'I',
            ï: 'i', Ï: 'I', i: 'i', I: 'I', j: 'j',
            J: 'J', k: 'k', K: 'K', l: 'l', L: 'L',
            n: 'n', N: 'N', ñ: 'n', Ñ: 'N', ó: 'o',
            Ó: 'O', ò: 'o', Ò: 'O', ô: 'o', Ô: 'O',
            o: 'o', O: 'O', õ: 'o', Õ: 'O', ø: 'oe',
            Ø: 'OE', ö: 'oe', Ö: 'OE', r: 'r', R: 'R',
            s: 's', S: 'S', š: 's', Š: 'S', ß: 'SS',
            t: 't', T: 'T', ú: 'u', Ú: 'U', ù: 'u',
            Ù: 'U', u: 'u', U: 'U', û: 'u', Û: 'U',
            ü: 'ue', Ü: 'UE', w: 'w', W: 'W', ý: 'y',
            Ý: 'Y', y: 'y', Y: 'Y', ÿ: 'y', Ÿ: 'Y',
            z: 'z', Z: 'Z', ž: 'z', Ž: 'Z', þ: 'th',
            Þ: 'Th', µ: 'u',
        };
        let newText = '';
        if (text) {
            for (let i = 0; i < text.length; i++) {
                newText += transliterationTable[text[i]] || text[i];
            }
        }
        return newText.replace(/ /g, '_').replace(/[^a-zA-Z0-9_-]/g, '').replace(/_+/i, '_');
    }

    public static deleteReport() {
        const form = Metadata.getInstance<Form>('template-form-report');
        const report = form.value.REPORT;
        if (report.isCustom) {
            DialogService.show({
                name: 'dialog-confirm-delete',
                type: 'warning',
                title: I18n.translate('CONFIRMATION'),
                text: I18n.translate('ALERT_CONFIRM_DELETE_REPORT_FILE'),
                buttons: [
                    {
                        name: 'button-no',
                        component: 'ZdButton',
                        label: I18n.translate('NO'),
                        events: {
                            click: () => DialogService.hide(),
                        },
                        outline: true,
                    },
                    {
                        name: 'button-yes',
                        component: 'ZdButton',
                        label: I18n.translate('YES'),
                        events: {
                            click: () => TekLibReport.deleteCustomReport(),
                        },
                    },
                ],
            });
        } else {
            Util.alertMessage('DELETE_REPORT_NOT_ALLOWED', 'error');
        }
    }

    public static deleteCustomReport() {
        const form = Metadata.getInstance<Form>('template-form-report');
        const report = form.value.REPORT;
        const params = {
            ID: report.id,
            FILE_NAME: report.value,
            WINDOW_NAME: form.userProperties.windowName,
        };
        LoadingService.show();
        Http.post('/lib_deleteCustomReport', params).then(() => {
            Util.showSuccessMessage();
            this.loadCustomReports();
            if (form.userProperties.reportDefaultValue) {
                const reportField = Metadata.getInstance<Select>('REPORT');
                reportField.value = form.userProperties.reportDefaultValue;
            }
        }).catch((error) => {
            Util.showErrorMessage(error);
        }).finally(() => {
            DialogService.hide();
            LoadingService.hide();
        });
    }

    public static async loadCustomReports() {
        const form = await Util.getAsyncFormInstance('template-form-report');
        const reportField = Metadata.getInstance<Select>('REPORT');
        const fixedReports = form.userProperties.fixedReports || [];
        const params = {
            WINDOW_NAME: form.userProperties.windowName,
        };
        Http.post('/lib_getCustomReports', params).then((response) => {
            const customReports = response.data.map((item: any) => {
                return {
                    id: item.ID,
                    value: item.FILE_NAME,
                    label: `${I18n.translate('CUSTOMIZED')} | ${item.LABEL}`,
                    parentReport: item.PARENT_REPORT,
                    isCustom: true,
                };
            });
            reportField.datasource.updateData(customReports.concat(fixedReports));
            if (!reportField.value && form.userProperties.reportDefaultValue) {
                reportField.value = form.userProperties.reportDefaultValue;
            }
        }).catch(Util.showErrorMessage);
    }

    public static async uploadModal(modal: Modal) {
        const modalForm = Metadata.getInstance<Form>('form-modal-report');
        const form = Metadata.getInstance<Form>('template-form-report');

        if (Util.validateForm(modalForm)) {
            modalForm.value.FILE_CONTENT = await Util.getFileBase64(modalForm.value.BIRT_FILE);
            modalForm.value.WINDOW_NAME = form.userProperties.windowName;
            LoadingService.show();
            Http.post('/lib_createCustomReport', modalForm.value).then(() => {
                modal.hide();
                TekLibReport.loadCustomReports();
                Util.showSuccessMessage();
            }).catch((error) => {
                Util.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
            });
        }
    }
}
